# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

certificate_help = """For a detailed explanation of CA certificates in Azure IoT Hub,
                      see https://docs.microsoft.com/azure/iot-hub/iot-hub-x509ca-overview """
# AAD application Id
CLIENT_ID = '822c8694-ad95-4735-9c55-256f7db2f9b4'
PNP_ENDPOINT = 'https://provider.azureiotrepository.com'
